﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "发票统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class InvoiceStatistic : RestfulExecution<Invoice>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public bool Offseted { get; set; }

        /// <summary>
        /// 用于查询借票是否到账使用 
        /// </summary>
        public bool? Payed { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Invoice>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.Type != "Relation");
            if (Offseted)//只查作废或者红字的发票
            {
                sqlExpress.And(o => o.OperateTime >= BeginTime && o.OperateTime < EndTime && o.Amount < 0);
            }
            else
            {
                //查当月的全部发票(包含当月到账的借票)
                sqlExpress.And(o => ((o.OperateTime >= BeginTime && o.OperateTime < EndTime) || (o.PayedTime.Value >= BeginTime && o.PayedTime.Value < EndTime && o.OperateTime < BeginTime)));
                if (Payed != null && Payed.Value) sqlExpress.And(o => o.AmountPayment == o.Amount);
                if (Payed != null && !Payed.Value) sqlExpress.And(o => o.AmountPayment < o.Amount);
            }
            var results = autoQuery.Execute(Request, conn, request, sqlExpress);

            var rtns = new StatisticResponse<Invoice>();
            rtns.PopulateWith(results);
            rtns.TotalAmount = rtns.Results.Sum(o => o.Amount);
            rtns.TotalAmountPayment = rtns.Results.Sum(o => o.AmountPayment);
            return rtns;
        }
    }

    [Api(Description = "结算统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SettlementStatistic : RestfulExecution<Settlement>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public bool? Payed { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Settlement>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            //sqlExpress.Where(o => o.OffsetKvid == Guid.Empty);
            if (Payed != null)
            {
                if (Payed.Value) sqlExpress.And(o => o.Amount == o.AmountPayment);
                else sqlExpress.And(o => o.Amount > o.AmountPayment);
            }
            sqlExpress.And(o => o.CreateTime >= BeginTime && o.CreateTime < EndTime);

            var results = autoQuery.Execute(Request, conn, request, sqlExpress);

            var rtns = new StatisticResponse<Settlement>();
            rtns.PopulateWith(results);
            rtns.TotalAmount = rtns.Results.Where(o => o.Amount >= 0).Sum(o => o.Amount);
            rtns.TotalQuantity = rtns.Results.Where(o => o.Amount >= 0).Count();
            rtns.TotalAmountPayment = rtns.Results.Where(o => o.Amount >= 0 && o.AmountPayment > 0).Sum(o => o.AmountPayment);
            rtns.TotalQuantityPayment = rtns.Results.Where(o => o.Amount >= 0 && o.AmountPayment == o.Amount).Count();
            rtns.TotalAmountUnpaid = rtns.Results.Where(o => o.Amount >= 0 && o.AmountPayment < o.Amount).Sum(o => o.Amount - o.AmountPayment);
            rtns.TotalQuantityUnpaid = rtns.Results.Where(o => o.Amount >= 0 && o.AmountPayment == 0).Count();
            rtns.TotalAmountOffset = rtns.Results.Where(o => o.Amount < 0).Sum(o => o.Amount);
            rtns.TotalQuantityOffset = rtns.Results.Where(o => o.Amount < 0).Count();
            return rtns;
        }
    }

    [Api(Description = "到账统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentStatistic : RestfulExecution<Payment>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public OwnerType OwnerType { get; set; } = OwnerType.Unsupport;

        public bool? IsPayee { get; set; }//查收款还是付款

        public bool? Used { get; set; }//到账是否已被使用

        public bool? Invoiced { get; set; }//到账是否开票

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Payment>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.WeChat, PaymentType.Cash, PaymentType.AliPay, PaymentType.Pos, PaymentType.Split, PaymentType.Refund, PaymentType.UnBiz));
            if (OwnerType == OwnerType.Organization) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.OrganizationKvid || o.OwnerKvid == Guid.Empty);
            if (OwnerType == OwnerType.Department) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.DepartmentKvid || o.OwnerKvid == Guid.Empty);
            if (OwnerType == OwnerType.Member) sqlExpress.And(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid || o.OwnerKvid == Guid.Empty);
            if (IsPayee != null)
            {
                var queryAccount = conn.From<Account>();
                queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
                queryAccount.Select(o => o.Kvid);

                if (IsPayee.Value) sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));//收款
                else sqlExpress.And(o => Sql.In(o.PayeeAccountKvid, queryAccount));//付款
            }
            if (Used != null)
            {
                if (Used.Value) sqlExpress.And(o => o.AmountUsed == o.Amount);
                else sqlExpress.And(o => o.AmountUsed < o.Amount);
            }
            if (Invoiced != null)
            {
                if (Invoiced.Value) sqlExpress.And(o => o.AmountInvoice == o.Amount);
                else sqlExpress.And(o => o.AmountInvoice < o.Amount);
            }

            var results = autoQuery.Execute(Request, conn, request, sqlExpress);
            var rtns = new StatisticResponse<Payment>();
            rtns.PopulateWith(results);
            rtns.TotalAmount = rtns.Results.Where(o => o.Amount >= 0).Sum(o => o.Amount);
            rtns.TotalQuantity = rtns.Results.Where(o => o.Amount >= 0).Count();
            rtns.TotalAmountInvoice = rtns.Results.Where(o => o.Amount >= 0 && o.AmountInvoice > 0).Sum(o => o.AmountInvoice);
            rtns.TotalQuantityInvoice = rtns.Results.Where(o => o.Amount >= 0 && o.AmountInvoice == o.Amount).Count();
            rtns.TotalAmountUsed = rtns.Results.Where(o => o.Amount >= 0 && o.AmountUsed > 0).Sum(o => o.AmountUsed);
            rtns.TotalQuantityUsed = rtns.Results.Where(o => o.Amount >= 0 && o.AmountUsed == o.Amount).Count();
            return rtns;
        }
    }

    public class StatisticResponse<T>:RestfulQueryResponse<T>
    {
        public decimal TotalAmount { get; set; }
        public decimal TotalAmountInvoice { get; set; }
        public decimal TotalAmountUsed { get; set; }
        public decimal TotalAmountPayment { get; set; }
        public decimal TotalAmountUnpaid { get; set; }
        public decimal TotalAmountOffset { get; set; }

        public decimal TotalQuantity { get; set; }
        public decimal TotalQuantityInvoice { get; set; }
        public decimal TotalQuantityUsed { get; set; }
        public decimal TotalQuantityPayment { get; set; }
        public decimal TotalQuantityUnpaid { get; set; }
        public decimal TotalQuantityOffset { get; set; }

    }
}
